'''
Created on 7 Jul 2021

@author: jacklok
'''

from flask import Blueprint, request, session 
from flask_restful import abort
import logging
from trexlib.utils.log_util import get_tracelog
from flask_restful import Api
from trexmodel.utils.model.model_util import create_db_client
from flask.json import jsonify
from datetime import datetime
from trexapi.decorators.api_decorators import auth_token_required,\
    outlet_key_required
from trexlib.utils.string_util import is_not_empty
from trexmodel.models.datastore.customer_models import Customer
from trexmodel.models.datastore.user_models import User
from trexadmin.libs.http import create_rest_message
from trexadmin.libs.http import StatusCode
from trexmodel.models.datastore.merchant_models import Outlet,\
    MerchantAcct
from trexapi.forms.customer_api_forms import CustomerDetailsForm
from werkzeug.datastructures import ImmutableMultiDict
import gettext

logger = logging.getLogger('api')


customer_api_bp = Blueprint('customer_api_base_bp', __name__,
                                 template_folder='templates',
                                 static_folder='static',
                                 url_prefix='/api/v1/customers')

logger = logging.getLogger('api')

customer_api = Api(customer_api_bp)



@customer_api_bp.route('/', methods=['POST'])
@auth_token_required
@outlet_key_required
def create_customer():
    customer_data_in_json   = request.get_json()
    register_customer_form  = CustomerDetailsForm(ImmutableMultiDict(customer_data_in_json))
    
    logger.debug('create_customer: customer_data_in_json=%s', customer_data_in_json)
    
    try:
        if register_customer_form.validate():
            logger.debug('customer registration input is valid')
            db_client = create_db_client(caller_info="create_customer")
            
            is_email_used           = False
            is_mobile_phone_used    = False
            customer_is_exist       = False
            created_customer        = None
            created_user_acct       = None
            
            with db_client.context():
                merchant_acct   = MerchantAcct.fetch(session.get('acct_id'))
                outlet          = Outlet.fetch(request.headers.get('x-outlet-key'))
                
                email           = customer_data_in_json.get('email')
                mobile_phone    = customer_data_in_json.get('mobile_phone')
                
                if is_not_empty(email):
                    created_user_acct = User.get_by_email(email)
                    
                    if created_user_acct:
                        is_email_used = True
                
                if is_not_empty(mobile_phone):
                    mobile_phone = mobile_phone.replace(" ", "")
                    created_user_acct = User.get_by_mobile_phone(mobile_phone)
                    
                    if created_user_acct:
                        is_mobile_phone_used = True
                
                
                logger.debug('is_email_used=%s', is_email_used)
                logger.debug('is_mobile_phone_used=%s', is_mobile_phone_used)
                
                if merchant_acct and outlet:
                        
                    logger.debug('merchant_acct.key_in_str=%s', merchant_acct.key_in_str)
                    logger.debug('outlet.merchant_acct_key=%s', outlet.merchant_acct_key)
                    
                    if merchant_acct.key_in_str == outlet.merchant_acct_key: 
                        logger.debug('Valid granted outlet key for merchant acct')
                        birth_date              = customer_data_in_json.get('birth_date')
                        
                        if is_not_empty(birth_date):
                            birth_date = datetime.strptime(birth_date, '%d/%m/%Y')
                        
                        logger.debug('birth_date=%s', birth_date)
                        
                        if is_email_used:
                            created_customer = Customer.get_by_email(email)
                            
                        elif is_mobile_phone_used:
                            created_customer = Customer.get_by_mobile_phone(mobile_phone)    
                        
                        if is_email_used or is_mobile_phone_used:
                            if created_customer is None:
                                logger.debug('User account have been created, but customer account is not yet created')
                                
                                created_user_acct = User.update(created_user_acct,
                                                                name                    = customer_data_in_json.get('name'), 
                                                                email                   = customer_data_in_json.get('email'), 
                                                                gender                  = customer_data_in_json.get('gender'),
                                                                birth_date              = birth_date,
                                                                mobile_phone            = customer_data_in_json.get('gender'), 
                                                                password                = customer_data_in_json.get('password'),
                                                                )
                                
                                created_customer        = Customer.create_from_user(merchant_acct, outlet, created_user_acct, customer_data_in_json.get('merchant_reference_code'))
                            
                            else:
                                customer_is_exist = True
                                logger.warn('Customer account using same email or mobile phone have been created')
                        else:
                            created_customer        = Customer.create(merchant_acct = merchant_acct, 
                                                            outlet                  = outlet, 
                                                            name                    = customer_data_in_json.get('name'), 
                                                            email                   = customer_data_in_json.get('email'), 
                                                            gender                  = customer_data_in_json.get('gender'),
                                                            birth_date              = birth_date,
                                                            mobile_phone            = mobile_phone, 
                                                            merchant_reference_code = customer_data_in_json.get('merchant_reference_code'), 
                                                            password                = customer_data_in_json.get('password'),
                                                            )
                        logger.debug('created_customer=%s', created_customer)
                    else:
                        logger.warn('Invalid granted outlet key')
                else:
                    logger.warn('Invalid granted outlet key')
                
                if customer_is_exist:
                    if is_email_used==True:
                        return create_rest_message('Email have been taken', status_code=StatusCode.BAD_REQUEST)
                    
                    elif is_mobile_phone_used==True:
                        return create_rest_message('Mobile phone have been taken', status_code=StatusCode.BAD_REQUEST)
                    
                else:
                    if created_customer:
                        return create_rest_message(status_code=StatusCode.OK, reference_code=created_customer.reference_code)
                    else:
                        return create_rest_message(status_code=StatusCode.BAD_REQUEST)
            
        else:
            logger.warn('customer registration input is invalid')
            error_message = register_customer_form.create_rest_return_error_message()
            
            return create_rest_message(error_message, status_code=StatusCode.BAD_REQUEST)
    except:
        logger.error('Fail to register customer due to %s', get_tracelog())
        
        return create_rest_message(status_code=StatusCode.BAD_REQUEST)


        

@customer_api_bp.route('/<ref_code>', methods=['GET'])
@auth_token_required
def read_customer(ref_code):
    
    logger.debug('ref_code=%s', ref_code)
    
    if is_not_empty(ref_code):
        db_client = create_db_client(caller_info="read_customer")
        with db_client.context():
            customer = Customer.get_by_reference_code(ref_code)
        
        if customer:
        
            customer_property_list = ['name', 'email', 'mobile_phone', 'gender', 'birth_date', 'merchant_reference_code']
            
            return jsonify(customer.to_dict(dict_properties=customer_property_list, show_key=False))
            
        else:
            logger.warn('Customer with reference code (%s) is not found', ref_code)
            return create_rest_message(status_code=StatusCode.BAD_REQUEST)
            
    else:
        return create_rest_message(status_code=StatusCode.BAD_REQUEST)


@customer_api_bp.route('/<ref_code>', methods=['PUT'])
@auth_token_required
def update_customer(ref_code):
    customer_data_in_json   = request.get_json()
    updating_customer_form  = CustomerDetailsForm(ImmutableMultiDict(customer_data_in_json))
    
    logger.debug('update_customer: customer_data_in_json=%s', customer_data_in_json)
    
    try:
        if updating_customer_form.validate():
            logger.debug('customer update input is valid')
            db_client = create_db_client(caller_info="update_customer")
            
            is_email_used                           = False
            is_mobile_phone_used                    = False
            customer_is_exist                       = False
            updating_customer                       = None
            checking_user_acct                      = None
            is_customer_email_changed               = False
            is_customer_mobile_phone_changed        = False
            is_used_email_same_user_acct            = False
            is_used_mobile_phone_same_user_acct     = False
            
            with db_client.context():
                email               = customer_data_in_json.get('email')
                mobile_phone        = customer_data_in_json.get('mobile_phone')
                
                if mobile_phone:
                    mobile_phone = mobile_phone.replace(" ", "")
                
                updating_customer   = Customer.get_by_reference_code(ref_code)
                
                if updating_customer:
                    customer_is_exist = True
                    is_customer_email_changed           = updating_customer.email!=email
                    is_customer_mobile_phone_changed    = updating_customer.mobile_phone!=mobile_phone
                
                if is_customer_email_changed:
                    if is_not_empty(email):
                        checking_user_acct = User.get_by_email(email)
                        
                        if checking_user_acct:
                            is_email_used = True
                            is_used_email_same_user_acct = checking_user_acct.key_in_str == updating_customer.registered_user_acct_key
                        
                if is_customer_mobile_phone_changed:
                    if is_not_empty(mobile_phone):
                        checking_user_acct = User.get_by_mobile_phone(mobile_phone)
                    
                        if checking_user_acct:
                            is_mobile_phone_used = True
                            is_used_mobile_phone_same_user_acct = checking_user_acct.key_in_str == updating_customer.registered_user_acct_key
                                
                logger.debug('is_customer_email_changed=%s', is_customer_email_changed)
                logger.debug('is_customer_mobile_phone_changed=%s', is_customer_mobile_phone_changed)
                logger.debug('is_email_used=%s', is_email_used)
                logger.debug('is_mobile_phone_used=%s', is_mobile_phone_used)
                logger.debug('is_used_email_same_user_acct=%s', is_used_email_same_user_acct)
                logger.debug('is_used_mobile_phone_same_user_acct=%s', is_used_mobile_phone_same_user_acct)
                
                if is_email_used==False and is_mobile_phone_used==False:
                    logger.debug('Going to update customer details') 
                    
                    birth_date              = customer_data_in_json.get('birth_date')
                        
                    if is_not_empty(birth_date):
                        birth_date = datetime.strptime(birth_date, '%d/%m/%Y')
                    
                    logger.debug('birth_date=%s', birth_date)
                    
                    Customer.update(customer=updating_customer, 
                                    name                    = customer_data_in_json.get('name'), 
                                    email                   = customer_data_in_json.get('email'), 
                                    gender                  = customer_data_in_json.get('gender'),
                                    birth_date              = birth_date,
                                    mobile_phone            = mobile_phone, 
                                    password                = customer_data_in_json.get('password'),
                                    )
                
                
                if customer_is_exist:
                    if is_email_used==True:
                        return create_rest_message('Email have been taken', status_code=StatusCode.BAD_REQUEST)
                    
                    elif is_mobile_phone_used==True:
                        return create_rest_message('Mobile phone have been taken', status_code=StatusCode.BAD_REQUEST)
                    else:
                        return create_rest_message(status_code=StatusCode.OK)
                else:
                    return create_rest_message(gettext('Customer is not exis'), status_code=StatusCode.BAD_REQUEST)
            
        else:
            logger.warn('customer registration input is invalid')
            error_message = updating_customer_form.create_rest_return_error_message()
            
            return create_rest_message(error_message, status_code=StatusCode.BAD_REQUEST)
    except:
        logger.error('Fail to register customer due to %s', get_tracelog())
        
        return create_rest_message(status_code=StatusCode.BAD_REQUEST)
        
@customer_api_bp.route('/<ref_code>', methods=['DELETE'])
@auth_token_required 
def delete_customer(ref_code):
    if is_not_empty(ref_code):
        is_found = False
        db_client = create_db_client(caller_info="read_customer")
        with db_client.context():
            customer = Customer.get_by_reference_code(ref_code) 
            if customer:
                customer.delete()
                is_found = True
        
        if is_found:
            return create_rest_message(status_code=StatusCode.NO_CONTENT)
        else:
            return create_rest_message(gettext('Customer is not exis'), status_code=StatusCode.BAD_REQUEST)
    else:
        return create_rest_message(status_code=StatusCode.BAD_REQUEST)        





#customer_api.add_resource(CustomerResource)    


'''
Created on 28 Oct 2021

@author: jacklok
'''

from flask import Blueprint 
import logging
from trexapi.decorators.api_decorators import auth_token_required
from trexlib.utils.string_util import is_not_empty, is_empty
from trexadmin.libs.http import create_rest_message
from trexadmin.libs.http import StatusCode
from trexmodel.models.datastore.pos_models import POSSetting
from trexmodel.utils.model.model_util import create_db_client
from flask.globals import request
from flask.helpers import url_for
from trexmodel.models.datastore.product_models import ProductCatalogue,\
    ProductCategory
from trexlib.utils.common.common_util import sort_list

#logger = logging.getLogger('api')
logger = logging.getLogger('debug')

pos_api_bp = Blueprint('pos_api_bp', __name__,
                                 template_folder='templates',
                                 static_folder='static',
                                 url_prefix='/api/v1/pos')

@pos_api_bp.route('/check-activation', methods=['POST'])
def check_activation():
    activation_code = request.args.get('activation_code') or request.form.get('activation_code') or request.json.get('activation_code')
    logger.debug('activation_code=%s', activation_code)
    
    if is_not_empty(activation_code):
        db_client = create_db_client(caller_info="check_activation")
        with db_client.context():
            pos_setting = POSSetting.get_by_activation_code(activation_code)
        
        if pos_setting:
            if pos_setting.activated==False:
                return create_rest_message(status_code=StatusCode.OK)
            else:
                return create_rest_message('The code have been used to activate before', status_code=StatusCode.BAD_REQUEST)
        else:
            return create_rest_message('Invalid activate code', status_code=StatusCode.BAD_REQUEST)
    else:
        return create_rest_message('Activation code is required', status_code=StatusCode.BAD_REQUEST)

def getPOSAccountSetting(activation_code):
    if is_not_empty(activation_code):
        db_client = create_db_client(caller_info="getPOSAccountSetting")
        with db_client.context():
            pos_setting = POSSetting.get_by_activation_code(activation_code)
        
        if pos_setting:
            logger.info('Found POS setting');
            #if pos_setting.activated==False:
            if True:
                logger.info('POS activation code is valid');
                pos_setting_details = None
                with db_client.context():
                    #is_activated = POSSetting.activate(activation_code)
                    #if is_activated:
                    
                    pos_setting_details                                 = pos_setting.setting
                    
                    pos_setting_details['logo_image_url']               = url_for('system_bp.merchant_logo_image_url', merchant_act_key=pos_setting_details.get('account_id'))
                
                #if is_activated:
                if True:
                    
                    logger.debug('pos_setting_details=%s', pos_setting_details);
                    
                    return create_rest_message(status_code=StatusCode.OK,
                                               **pos_setting_details
                                               )
                else:
                    return create_rest_message('Failed to activate', status_code=StatusCode.BAD_REQUEST)
            else:
                logger.info('POS activation code have been used');
                return create_rest_message('The code have been used to activate before', status_code=StatusCode.BAD_REQUEST)
        else:
            logger.info('POS setting record is not found');
            return create_rest_message('Invalid activate code', status_code=StatusCode.BAD_REQUEST)
    else:
        logger.info('activation_code is empty');
        return create_rest_message('Activation code is required', status_code=StatusCode.BAD_REQUEST)
    
    
@pos_api_bp.route('/activate', methods=['POST'])
@pos_api_bp.route('/account-sync', methods=['get'])
def activate():
    activation_code = request.args.get('activation_code') or request.form.get('activation_code') or request.json.get('activation_code')
    
    logger.debug('activation_code=%s', activation_code)
    return getPOSAccountSetting(activation_code)
       
    
def parse_category_to_json(category_tree_structure):
    data_list = []
    
    for category in category_tree_structure:
        data = {
                    'code'                  : category.get('key'),
                    'label'                 : category.get('category_label'),
                    'label_with_item_count' : category.get('category_label_and_other_details'),
                    'group'                 : category.get('has_child'),
                    'product_modifier'      : category.get('product_modifier'),
                    'product_items'         : category.get('product_items'),
                }
        if category.get('childs'):
            child_data_list = parse_category_to_json(category.get('childs'))
            if child_data_list:
                data['childs'] = child_data_list   
        
        data_list.append(data)
    
    return data_list

def construct_category_tree_structure(category_tree_structure, category_list):
    for category in category_list:
        if is_empty(category.get('parent_category_key')):
            #top most category
            category_tree_structure.append(category)
            __find_child_category(category, category_list)
                

def __lookup_category_from_category_list(category_code, category_list):
    for category in category_list:
        if category.get('key') == category_code:
            return category  

    
def __find_child_category(category, category_list):
    if is_not_empty(category.get('child_category_keys')):
        childs                      = []
        parent_product_modifier     = category.get('product_modifier') or []
        
        for child_category_key in category.get('child_category_keys'):
            child = __lookup_category_from_category_list(child_category_key, category_list)
            
            logger.debug('category product_modifier of %s =%s', category.get('key'), parent_product_modifier)
            if child:
                child_product_modifier      = child.get('product_modifier') or []
                child_product_modifier      = list(set(parent_product_modifier) | set(child_product_modifier) )
                child['product_modifier']   = child_product_modifier
                
                logger.debug('child_product_modifier of %s =%s', category.get('key'), child_product_modifier)
                
                if is_not_empty(child.get('child_category_keys')):
                    __find_child_category(child, category_list)
                childs.append(child)
        
        category['childs'] = childs

def get_product_category_structure_code_label_json(merchant_acct):
    category_list       = get_product_category_structure(merchant_acct)
    
    category_tree_structure = []
    
    construct_category_tree_structure(category_tree_structure, category_list)
    
    return parse_category_to_json(category_tree_structure)

def get_product_category_structure(merchant_acct):
    
    sorted_category_structure = []
    
    category_structure      = ProductCategory.get_structure_by_merchant_acct(merchant_acct)
    category_structure      = sort_list(category_structure, sort_attr_name='category_label')
    
    #logger.debug('category_structure=%s', category_structure)
    
    for c in category_structure:
        sorted_category_structure.append(c.to_dict())
    
    
    logger.debug('sorted_category_structure=%s', sorted_category_structure)
            
    return sorted_category_structure

@pos_api_bp.route('/check-catalogue-status', methods=['GET'])
@auth_token_required
def check_catalogue_status():
    activation_code = request.args.get('activation_code') or request.form.get('activation_code') or request.json.get('activation_code')
    
    if is_not_empty(activation_code):
        pos_setting = None
        db_client = create_db_client(caller_info="check_catalogue_status")
        with db_client.context():
            pos_setting = POSSetting.get_by_activation_code(activation_code)
            if pos_setting:
                
                assigned_outlet = pos_setting.assigned_outlet_entity
                
                catalogue_key   = assigned_outlet.assigned_catalogue_key
                
                if is_not_empty(catalogue_key):
                    product_catalogue   = ProductCatalogue.fetch(catalogue_key)
                    if product_catalogue:
                        return {
                                    'last_updated_datetime' : product_catalogue.modified_datetime.strftime('%d-%m-%Y %H:%M:%S')
                                } 
                    else:
                        return create_rest_message('Invalid catalogue data', status_code=StatusCode.BAD_REQUEST)
        
        if pos_setting is None:
            return create_rest_message('Invalid activation code', status_code=StatusCode.BAD_REQUEST)
        else:
            return create_rest_message('Not catqalogue have been assigned or published', status_code=StatusCode.BAD_REQUEST)
                
    else:
        return create_rest_message('Activation code is required', status_code=StatusCode.BAD_REQUEST)
    
@pos_api_bp.route('/get-catalogue', methods=['GET'])
@auth_token_required
def get_catalogue():
    activation_code = request.args.get('activation_code') or request.form.get('activation_code') or request.json.get('activation_code')
    
    logger.debug('activation_code=%s', activation_code);
    
    if is_not_empty(activation_code):
        pos_setting         = None
        db_client           = create_db_client(caller_info="get_catalogue")
        valid_return        = None
        product_catalogue   = None
        with db_client.context():
            pos_setting = POSSetting.get_by_activation_code(activation_code)
            if pos_setting:
                
                assigned_outlet = pos_setting.assigned_outlet_entity
                
                logger.info('assigned_outlet name=%s', assigned_outlet.name)
                
                catalogue_key   = assigned_outlet.assigned_catalogue_key
                
                logger.debug('catalogue_key=%s', catalogue_key);
                
                if is_not_empty(catalogue_key):
                    product_catalogue   = ProductCatalogue.fetch(catalogue_key)
                    merchant_acct       = pos_setting.merchant_acct_entity
                    
                    #category_tree_structure_in_json  = json.dumps(get_product_category_structure_code_label_json(merchant_acct), sort_keys = True, separators = (',', ': '))
                    category_tree_structure_in_json  = get_product_category_structure_code_label_json(merchant_acct)
                    
                    if product_catalogue:
                        last_updated_datetime = product_catalogue.modified_datetime
                        if assigned_outlet.modified_datetime is not None and assigned_outlet.modified_datetime>last_updated_datetime:
                            last_updated_datetime = assigned_outlet.modified_datetime
                        '''
                        dinning_option_json = []
                        dinning_option_list = DinningOption.list_by_merchant_acct(merchant_acct)
                        
                        
                        if dinning_option_list:
                            for d in dinning_option_list:
                                dinning_option_json.append({
                                                            'option_key'                : d.key_in_str,
                                                            'option_name'               : d.name,
                                                            'is_default'                : d.is_default,
                                                            'dinning_table_is_required' : d.dinning_table_is_required,
                                                            'assign_queue'              : d.assign_queue,
                                                            })
                        '''        
                        valid_return =  {
                                                'category'              : category_tree_structure_in_json,
                                                'menu'                  : product_catalogue.published_menu_settings,
                                                #'dinning_table_list'    : assigned_outlet.assigned_dinning_table_list,
                                                #'dinning_option_list'   : dinning_option_json,
                                                'last_updated_datetime' : last_updated_datetime.strftime('%d-%m-%Y %H:%M:%S')
                                            } 
                    
                
                    
        if valid_return:
            return valid_return
        else:
            if product_catalogue==None:
                logger.debug('No catalogue have been assigned or published');
                return create_rest_message('No catqalogue have been assigned or published', status_code=StatusCode.BAD_REQUEST)
            else:
                if pos_setting is None:
                    logger.debug('Invalid activation code');
                    return create_rest_message('Invalid activation code', status_code=StatusCode.BAD_REQUEST)
                else:
                    logger.debug('No catalogue have been assigned or published');
                    return create_rest_message('No catqalogue have been assigned or published', status_code=StatusCode.BAD_REQUEST)
                
    else:
        return create_rest_message('Activation code is required', status_code=StatusCode.BAD_REQUEST)    

@pos_api_bp.route('/version-sync', methods=['get'])
def version_sync():
    db_client       = create_db_client(caller_info="version_sync")
    
    with db_client.context():
        version =  {
                                '''
                                'setting':[
                                            {
                                                "table_name": "setting",
                                                "version" : 1,
                                                "script": "ALTER TABLE setting ADD COLUMN account_settings TEXT",
                                            }
                                               
                                        ],
                                'user'  :[
                                            {
                                                "table_name": "user",
                                                "version" : 1,
                                                "script": "ALTER TABLE user ADD COLUMN token_expiry_datetime TEXT",
                                                
                                            }
                                        ]
                                '''        
        
                                'cart'  :[
                                            {
                                                "table_name": "cart",
                                                "version" : 1,
                                                "script": "ALTER TABLE invoice ADD COLUMN void_by TEXT",
                                                
                                            }
                                        ]
                            }
                                
    
            
    
    return version


     
           
